@@ -48,11 +48,6 @@ in this Software without prior written authorization from The Open Group.
 #include <X11/keysymdef.h>
 #include <stdio.h>
 
-#ifdef USE_OWN_COMPOSE
-#include "imComp.h"
-
-#endif
-
 #include "Xresource.h"
 #include "Key.h"
 
@@ -276,12 +271,13 @@ _XKeyInitialize(
 	if (! keysyms) return 0;
 
 	LockDisplay(dpy);
-	if (dpy->keysyms)
-	    Xfree (dpy->keysyms);
+
+	Xfree (dpy->keysyms);
 	dpy->keysyms = keysyms;
 	dpy->keysyms_per_keycode = per;
 	if (dpy->modifiermap)
 	    ResetModMap(dpy);
+
 	UnlockDisplay(dpy);
     }
     if (!dpy->modifiermap)
@@ -891,73 +887,6 @@ XLookupString (
 		  &modifiers, &symbol))
 	return 0;
 
-#ifdef USE_OWN_COMPOSE
-    if ( status ) {
-	static int been_here= 0;
-	if ( !been_here ) {
-	    XimCompInitTables();
-	    been_here = 1;
-	}
-	if ( !XimCompLegalStatus(status) ) {
-	    status->compose_ptr = NULL;
-	    status->chars_matched = 0;
-	}
-	if ( ((status->chars_matched>0)&&(status->compose_ptr!=NULL)) ||
-		XimCompIsComposeKey(symbol,event->keycode,status) ) {
-	    XimCompRtrn rtrn;
-	    switch (XimCompProcessSym(status,symbol,&rtrn)) {
-		case XIM_COMP_IGNORE:
-		    break;
-		case XIM_COMP_IN_PROGRESS:
-		    if ( keysym!=NULL )
-			*keysym = NoSymbol;
-		    return 0;
-		case XIM_COMP_FAIL:
-		{
-		    int n = 0, len= 0;
-		    for (n=len=0;rtrn.sym[n]!=XK_VoidSymbol;n++) {
-			if ( nbytes-len > 0 ) {
-			    len+= _XTranslateKeySym(event->display,rtrn.sym[n],
-							event->state,
-							buffer+len,nbytes-len);
-			}
-		    }
-		    if ( keysym!=NULL ) {
-			if ( n==1 )	*keysym = rtrn.sym[0];
-			else		*keysym = NoSymbol;
-		    }
-		    return len;
-		}
-		case XIM_COMP_SUCCEED:
-		{
-		    int len,n = 0;
-
-		    symbol = rtrn.matchSym;
-		    if ( keysym!=NULL )	*keysym = symbol;
-		    if ( rtrn.str[0]!='\0' ) {
-			strncpy(buffer,rtrn.str,nbytes-1);
-			buffer[nbytes-1]= '\0';
-			len = strlen(buffer);
-		    }
-		    else {
-			len = _XTranslateKeySym(event->display,symbol,
-							event->state,
-							buffer,nbytes);
-		    }
-		    for (n=0;rtrn.sym[n]!=XK_VoidSymbol;n++) {
-			if ( nbytes-len > 0 ) {
-			    len+= _XTranslateKeySym(event->display,rtrn.sym[n],
-							event->state,
-							buffer+len,nbytes-len);
-			}
-		    }
-		    return len;
-		}
-	    }
-	}
-    }
-#endif
-
     if (keysym)
 	*keysym = symbol;
     /* arguable whether to use (event->state & ~modifiers) here */
@@ -1001,8 +930,8 @@ XRebindKeysym (
 	((! (p->string = Xmalloc(nbytes))) && (nbytes > 0)) ||
 	((! (p->modifiers = Xmalloc(nb))) && (nb > 0))) {
 	if (p) {
-	    if (p->string) Xfree(p->string);
-	    if (p->modifiers) Xfree(p->modifiers);
+	    Xfree(p->string);
+	    Xfree(p->modifiers);
 	    Xfree(p);
 	}
 	UnlockDisplay(dpy);
