@@ -33,7 +33,7 @@ THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 #define NEED_MAP_READERS
 #include "Xlibint.h"
-#include <X11/extensions/XKBgeom.h>
+#include "X11/extensions/XKBgeom.h"
 #include <X11/extensions/XKBproto.h>
 #include "XKBlibint.h"
 
